@@ -33,6 +33,9 @@ module Data.Marc21 (
     , getSubFieldValueList
     , addVarField
         
+    -- * Simple pretty printing
+    , dumpLines      -- :: Record -> [ByteString]
+    , putLines       -- :: Record -> IO ()
     ) where
 
 import           Data.ByteString (ByteString)
@@ -196,4 +199,24 @@ newDirData t nd dir os accFld  = (newDir,newOffset,newAccFld)
           osFmtd      = justifyRight 5 '0' (C.pack (show os))
           newDir      = dir <%> t <%> dataLenFmtd <%> osFmtd
           newAccFld   = accFld <%> nd
-    
+
+-- | Dump the Record into ByteString lines, subfields taking one line each.
+--
+-- This does not alter character set encodings; Unicode would be nice.
+dumpLines :: Record -> [ByteString]
+dumpLines r = (C.pack "LDR     " <> ldr r) : foldr prepend [] (fields r)
+  where
+    prepend vf@(VarField _ _ _ sfs) ls = case sfs of
+        []     -> introduce vf : ls  -- VarField has no subfields! (Illegal!)
+        s : ss -> (introduce vf <> C.pack "  " <> render s) :
+                  (more ss ++ ls)
+    prepend cf                      ls = introduce cf : ls
+    introduce (VarField t i1 i2 _) = t <> C.pack " " <> C.pack [i1, i2]
+    introduce (CntlField t d)      = t <> C.pack "     " <> d
+    more []         = []
+    more (sf : sfs) = (C.pack "        " <> render sf) : more sfs
+    render (sfc, value) = C.cons '_' (sfc <> value)
+
+-- | Using dumpLines, put Record into stdout.
+putLines :: Record -> IO ()
+putLines r = foldMap C.putStrLn (dumpLines r)
